import os
import subprocess
import tempfile
from pathlib import Path

from loguru import logger

from ..commands.meta_merger import MetaMerger
from ..model.test_tool import TestTool
from .testtool import github_asset_gen


class PullRequestGenerator:
    def __init__(self, testtool: TestTool):
        self.testtool = testtool

    def merge_and_create_pull_request(self) -> None:
        if "GH_TOKEN" not in os.environ:
            raise ValueError("Missing GH_TOKEN environment variable")

        with tempfile.TemporaryDirectory() as temp_dir:
            args = ["gh", "repo", "clone", "OpenTestSolar/testtool-registry", temp_dir]
            logger.debug(f"Running command: {' '.join(args)}")
            subprocess.run(
                args,
                check=True,
            )
            git_dir = Path(temp_dir)
            branch_name = f"testtools/{self.testtool.lang}/{self.testtool.name}/{self.testtool.version}"
            subprocess.run(
                [
                    "git",
                    "checkout",
                    "-b",
                    branch_name,
                ],
                cwd=git_dir,
                check=True,
            )

            merger = MetaMerger(self.testtool, asset_url_gen=github_asset_gen)
            merger.merge_index_and_history(git_dir)

            subprocess.run(["git", "add", "."], cwd=git_dir, check=True)
            subprocess.run(
                [
                    "git",
                    "commit",
                    "-m",
                    f"release {branch_name}",
                ],
                cwd=git_dir,
                check=True,
            )
            token = os.environ["GH_TOKEN"]
            subprocess.run(
                [
                    "git",
                    "remote",
                    "set-url",
                    "origin",
                    f"https://{token}@github.com/OpenTestSolar/testtool-registry.git",
                ],
                cwd=git_dir,
                check=True,
            )
            subprocess.run(
                ["git", "push", "--set-upstream", "origin", branch_name],
                cwd=git_dir,
                check=True,
            )
            subprocess.run(
                [
                    "gh",
                    "pr",
                    "create",
                    "--title",
                    f"🤖Release {branch_name}",
                    "--body",
                    f"发布测试工具 {self.testtool.name} 版本 {self.testtool.version}",
                    "--base",
                    "main",
                ],
                cwd=git_dir,
                check=True,
            )



import os
from pathlib import Path
from typing import Optional

from loguru import logger

from ..model.test_tool import StableIndexMetaData, MetaDataHistory


class ToolValidator:
    def __init__(self, workdir: Optional[str]) -> None:
        if workdir:
            self.workdir = Path(workdir)
        else:
            self.workdir = Path.cwd()

    def validate(self) -> None:
        """
        检查json文件是否符合要求
        """

        self.validate_stable_index()
        self.validate_tool_meta_json()

    def validate_stable_index(self) -> None:
        stable_index_file = Path(self.workdir) / "testtools" / "stable.index.json"
        logger.info(f"Validating stable index file [{stable_index_file}]")

        with open(stable_index_file) as f:
            sim = StableIndexMetaData.model_validate_json(f.read())

            logger.info(f"✅ Validated stable index file [{stable_index_file}] OK.")
            logger.info(f"✅ It has {len(sim.tools)} tools.")

    def validate_tool_meta_json(self) -> None:
        for dir_path, _, filenames in os.walk(self.workdir / "testtools"):
            for filename in filenames:
                if filename != "stable.index.json":
                    metafile = Path(dir_path) / filename
                    logger.info(f"Validating tool meta file [{metafile}]")
                    with open(metafile) as f:
                        re = MetaDataHistory.model_validate_json(f.read())
                        if re.versions:
                            # 检查versions中是否有重复版本
                            all_versions = set(x.meta.version for x in re.versions)

                            if len(all_versions) != len(re.versions):
                                raise RuntimeError(
                                    f"去重之后的版本数目 [{len(all_versions)}] != 原始版本数目 [{len(re.versions)}]"
                                )

                            logger.info(
                                f"✅ Validated tool [{re.versions[0].meta.name}] OK."
                            )



from pathlib import Path

import pytest

from solar_registry.service.pr_generator import PullRequestGenerator
from solar_registry.service.testtool import get_testtool


@pytest.mark.skip(reason="跳过真正合并github文件流程")
def test_merge_and_create_pull_request() -> None:
    workdir = str(Path(__file__).parent.joinpath("testdata").resolve())
    testtool = get_testtool(tool_name="pytest", workdir=workdir)

    gen = PullRequestGenerator(testtool)
    gen.merge_and_create_pull_request()



import os
from pathlib import Path
from typing import Optional, Tuple, List

import yaml
from loguru import logger

from ..model.test_tool import TestTool


def get_testtool(tool_name: str, workdir: Optional[str]) -> TestTool:
    logger.debug(f"querying testtool for {tool_name}")
    workdir = workdir or os.getcwd()
    return get_testtool_by_file_path(Path(workdir) / tool_name / "testtool.yaml")


def get_testtool_by_file_path(file_path: Path) -> TestTool:
    logger.debug(f"querying testtool for {file_path}")
    return _parse_testtool(file_path, strict=True)


def _parse_testtool(yaml_file: Path, strict: bool) -> TestTool:
    with open(yaml_file) as f:
        context = {}
        if strict:
            context["strict"] = True

        testtool = TestTool.model_validate(yaml.safe_load(f), context=context)
        logger.debug(
            f"loaded testtool: {testtool.model_dump_json(by_alias=True, indent=2, exclude_none=True)}"
        )
        return testtool


def github_asset_gen(testtool: TestTool) -> str:
    if not testtool.repository:
        raise ValueError(f"repository is required in testtool: {testtool.model_dump()}")
    repo = testtool.repository.rstrip("/")
    return f"{repo}/archive/refs/tags/{testtool.version}.tar.gz"


def sort_test_tools(tools: List[TestTool]) -> List[TestTool]:
    sorted_tools = sorted(list(enumerate(tools)), key=_sort_key)
    return [tool for index, tool in sorted_tools]


def _sort_key(item: Tuple[int, TestTool]) -> Tuple[int, int]:
    """
    按照优先级对其进行排序，如果没有优先级则保持原有排序

    如果 priority 为 None，返回 (0, index)，其中 index 是元素在原列表中的索引。
    如果 priority 不为 None，返回 (1, -priority)，这样 priority 越大，排序值越小，顺序越靠前。
    """
    index, tool = item
    if tool.priority is None:
        return 1, index
    else:
        return 0, -tool.priority



from enum import Enum
from typing import Literal, Optional

from pydantic import BaseModel, Field


class TestType(str, Enum):
    AutoTest = "auto-test"  # 一般自动化测试
    LoadTest = "load-test"  # 压力测试


class ResourceType(str, Enum):
    Node = "node"
    Android = "android"
    IOS = "ios"


class MetaExtraField(BaseModel):
    cli: Optional[str] = None


class ResourceDefaultConfig(BaseModel):
    name: str
    remark: str
    type: str
    group: str
    max_cnt: str = Field(alias="maxCnt")
    min_cnt: str = Field(alias="minCnt")
    condition: str
    parallel: str
    image: str


class PropertySpec(BaseModel):
    widget: str
    description: str
    name: str
    default: Optional[str] = None
    display_name: str = Field(alias="displayName")
    type: str
    choices: Optional[dict[str, str]] = None


class TestProfile(BaseModel):
    name: str
    description: str
    resource_types: list[ResourceType] = Field(alias="resourceTypes")
    resource_default_configs: list[ResourceDefaultConfig] = Field(
        alias="resourceDefaultConfigs"
    )
    property_specs: list[PropertySpec] = Field(alias="propertySpecs")
    job_template_xml: Optional[str] = Field(None, alias="jobTemplateXml")
    job_template_yaml: Optional[str] = Field(None, alias="jobTemplateYaml")


class LegacySpec(BaseModel):
    testcase_runner: MetaExtraField = Field(alias="testcaseRunner")
    testcase_loader: MetaExtraField = Field(alias="testcaseLoader")
    testcase_analyzer: MetaExtraField = Field(alias="testcaseAnalyzer")
    scaffolding_tool: MetaExtraField = Field(alias="scaffoldingTool")
    node_setup: MetaExtraField = Field(alias="nodeSetup")
    node_cleanup: MetaExtraField = Field(alias="nodeCleanup")
    global_setup: MetaExtraField = Field(alias="globalSetup")
    global_cleanup: MetaExtraField = Field(alias="globalCleanup")
    report_type: Literal["qta", "junit"] = Field(alias="reportType")
    res_pkg_url: str = Field(alias="resPkgUrl")
    doc_url: str = Field(alias="docUrl")
    logo_img_url: str = Field(alias="logoImgUrl")
    enable_code_coverage: bool = Field(alias="enableCodeCoverage")
    maintainers: list[str] = Field(alias="maintainers")
    test_type: TestType = Field(TestType.AutoTest, alias="testType")
    test_profiles: Optional[list[TestProfile]] = Field(None, alias="testProfiles")



from pathlib import Path

import pytest
from pydantic import ValidationError
from rich import print

from solar_registry.model.test_tool import OsType, ArchType, ParamWidget, TestTool
from solar_registry.service.testtool import (
    get_testtool,
    _parse_testtool,
    get_testtool_by_file_path,
    sort_test_tools,
)


def test_validate_correct_pytest_tool() -> None:
    workdir = str((Path(__file__).parent / "testdata").resolve())

    tool = get_testtool("pytest", workdir)

    assert tool.name == "pytest"
    assert tool.version == "0.1.6"
    assert tool.name_zh == "pytest自动化测试"
    assert tool.git_pkg_url == "github.com/OpenTestSolar/testtool-python@main:pytest"
    assert (
        tool.http_pkg_url
        == "https://opentestsolar.github.io/testtool-registry/testtools/python/pytest/metadata.json@stable"
    )
    assert tool.repository == "https://github.com/OpenTestSolar/testtool-python-pytest"

    assert tool.param_defs
    assert tool.param_defs[0].choices
    assert tool.param_defs[0].choices[0].display_name == "自动解析模式"
    assert tool.param_defs[0].qta_name == "parse_mode"

    assert tool.param_defs[3].input_widget == ParamWidget.Switch

    assert tool.support_os
    assert tool.support_os[0] == OsType.Windows
    assert tool.support_os[1] == OsType.Linux
    assert tool.support_os[2] == OsType.Darwin

    assert tool.support_arch
    assert tool.support_arch[0] == ArchType.Amd64
    assert tool.support_arch[1] == ArchType.Arm64

    assert tool.supported_certified_images
    assert len(tool.supported_certified_images) == 4
    assert tool.supported_certified_images[0] == "python:3.9"

    assert tool.test_catalog == "unit"
    assert tool.test_domains == ["windows", "macos", "server"]


def test_validate_loose() -> None:
    _parse_testtool(
        Path(__file__).parent
        / "testdata"
        / "error_meta_files"
        / "error_yaml_with_loose.yaml",
        strict=False,
    )


def test_validate_strict_with_errors() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(
            Path(__file__).parent
            / "testdata"
            / "error_meta_files"
            / "error_yaml_with_strict.yaml",
            strict=True,
        )

    print(ve.value)

    assert r"String should match pattern '^[a-zA-Z-0-9_]+$'" in str(ve.value)
    assert r"Input should be 'COMPILED' or 'INTERPRETED'" in str(ve.value)
    assert r"Input should be 'python', 'golang', 'javascript', 'java' or 'cpp'" in str(
        ve.value
    )
    assert r"String should match pattern '^(\d+\.\d+\.\d+|stable)$'" in str(ve.value)
    assert r"Input should be 'linux', 'windows', 'darwin' or 'android'" in str(ve.value)
    assert r"Input should be 'amd64' or 'arm64'" in str(ve.value)


def test_validate_strict_with_no_os() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(
            Path(__file__).parent
            / "testdata"
            / "error_meta_files"
            / "error_yaml_with_strict_no_os.yaml",
            strict=True,
        )

    print(ve.value)
    assert r"Value error, supportOS must be set" in str(ve.value)


def test_validate_strict_with_no_arch() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(
            Path(__file__).parent
            / "testdata"
            / "error_meta_files"
            / "error_yaml_with_strict_no_arch.yaml",
            strict=True,
        )

    print(ve.value)
    assert r" Value error, need at least 1 support arch" in str(ve.value)


def test_parse_legacy_tool() -> None:
    tool = _parse_testtool(
        Path(__file__).parent / "testdata" / "legacy" / "testtool.yaml", strict=True
    )

    assert tool.name == "qt4s_pot-line"
    assert not tool.git_pkg_url
    assert tool.param_defs
    param1 = tool.param_defs[0]
    assert param1.name == "setup_cmdline"
    assert param1.value == "初始化命令"
    assert param1.desc == "加载/执行用例前的初始化命令"
    assert param1.default == "# place your extra init command here"
    assert param1.lang == "bash"
    assert param1.input_widget == ParamWidget.Code

    assert tool.legacy_spec
    assert tool.legacy_name == "qt4s"
    assert tool.legacy_spec.report_type == "junit"
    assert tool.legacy_spec.maintainers == ["aa", "bb"]
    assert tool.legacy_spec.testcase_runner.cli == "test"
    assert tool.legacy_spec.testcase_loader.cli == "test"
    assert tool.legacy_spec.testcase_analyzer.cli == "test"
    assert tool.legacy_spec.scaffolding_tool.cli == "test"
    assert tool.legacy_spec.node_setup.cli == "test"
    assert tool.legacy_spec.node_cleanup.cli == "test"
    assert tool.legacy_spec.global_setup.cli == "test"
    assert tool.legacy_spec.global_cleanup.cli == "test"
    assert tool.legacy_spec.res_pkg_url == "https://sample.com/pkg"
    assert tool.legacy_spec.doc_url == "https://sample.com/doc"
    assert tool.legacy_spec.logo_img_url == "https://sample.com/logo"
    assert not tool.legacy_spec.enable_code_coverage
    assert tool.legacy_spec.test_type == "auto-test"


def test_get_testtool_by_file_path() -> None:
    tool = get_testtool_by_file_path(
        Path(__file__).parent / "testdata" / "legacy" / "testtool.yaml"
    )
    assert tool.name == "qt4s_pot-line"


def test_sort_testtools_by_priority() -> None:
    tools = [
        TestTool(
            schemaVersion=1,
            name="a",
            defaultBaseImage="a",
            description="测试数据测试数据测试数据",
            homePage="a",
            indexFile="a",
            lang="python",
            langType="INTERPRETED",
            version="0.1.1",
            versionFile="a",
            priority=1,
        ),
        TestTool(
            schemaVersion=1,
            name="b",
            defaultBaseImage="a",
            description="测试数据测试数据测试数据",
            homePage="a",
            indexFile="a",
            lang="python",
            langType="INTERPRETED",
            version="0.1.1",
            versionFile="a",
            priority=200,
        ),
        TestTool(
            schemaVersion=1,
            name="c",
            defaultBaseImage="a",
            description="测试数据测试数据测试数据",
            homePage="a",
            indexFile="a",
            lang="python",
            langType="INTERPRETED",
            version="0.1.1",
            versionFile="a",
        ),
        TestTool(
            schemaVersion=1,
            name="d",
            defaultBaseImage="a",
            description="测试数据测试数据测试数据",
            homePage="a",
            indexFile="a",
            lang="python",
            langType="INTERPRETED",
            version="0.1.1",
            versionFile="a",
        ),
    ]

    sorted_tools = sort_test_tools(tools)
    assert sorted_tools[0].name == "b"
    assert sorted_tools[1].name == "a"
    assert sorted_tools[2].name == "c"
    assert sorted_tools[3].name == "d"
